/*****************************************************************
 * Copyright (c) 2016 Texas Instruments and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Paul Gingrich - Initial API and implementation
 *****************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};
gc.databind.internal = gc.databind.internal || {};

gc.databind.internal.JsonCodec = function()
{
    gc.databind.AbstractPacketCodec.call(this);
    
    this.partialJsonMessage = '';
    this.numPacketsReceived = 0;
};

gc.databind.internal.JsonCodec.prototype = new gc.databind.AbstractPacketCodec();

gc.databind.internal.JsonCodec.prototype.encode = function(value) 
{
    return JSON.stringify(value) + '\n';
};

gc.databind.internal.JsonCodec.prototype.decode = function(rawdata, streamingModel)
{
    try
    {
        var message = String.fromCharCode.apply(null, rawdata);
        var packets = (this.partialJsonMessage + message).split("\n");
        var size = packets.length - 1;
        for (var i = 0; i < size; i++)
        {
            var packet = packets[i];
            var cleanPacket = "";
            try
            {
                // remove any leading or trailing garbage characters
                cleanPacket = packet.substring(packet.indexOf('{'), packet.lastIndexOf('}') + 1);
                // remove any spaces between : and the value
                while (cleanPacket.indexOf(': ') > 0)
                {
                    cleanPacket = cleanPacket.substring(0, cleanPacket.indexOf(': ') + 1) + cleanPacket.substring(cleanPacket.indexOf(': ') + 2).trim();
                }
                streamingModel.receiveData(JSON.parse(cleanPacket));
                this._isConnected = true;
            }
            catch(e)
            {
                this._isConnected = false;
                if (this.numPacketsReceived > 0) 
                {
                    console.log('JsonCodec: received non JSON data string:[' + cleanPacket + ']');
                }
            }
            if (this.numPacketsReceived === 0) 
            {
                console.log("Put breakpoint here to debug problems with serial I/O bindings");
            }
            this.numPacketsReceived++;
        }
        this.partialJsonMessage = packets[size];
    }
    catch(ex)
    {
        console.log('JsonCodec: Exception converting buffer to text string');
        this._isConnected = false;
    }
};
